/*************************************************************************************/
/* File lcd.h                                                                        */
/* Ansteuerung LC-Display fr Batronix Experimentierboard mit T89C51AC2 und          */
/* LC-Display "Displaytech C162" (HD44780-Controller)                                */
/*                                                                                   */
/* Autor: Markus Kerber                                                              */
/* Version: 1                                                                        */
/*************************************************************************************/

#ifndef _LCD_H_
#define _LCD_H_

#define _LCD_DDRAM 0x80           /* Adresse DDRAM */
#define _LCD_CGRAM 0x40           /* Adresse CGRAM */

#define _LCD_Row0 0x00             /* Offset 1. Zeile */
#define _LCD_Row1 0x40             /* Offset 2. Zeile */

#define _LCD_BLINK_CURSOR 0x0F    /* Display an, Cursor an, blinken an */
#define _LCD_SHIFT_RIGHT 0x1C     /* bewegt Displayinhalt nach rechts */
#define _LCD_SHIFT_LEFT 0x18      /* bewegt Displayinhalt nach links */

#define _LCD_CURSOR_RIGHT 0x14    /* bewegt den Cursor nach rechts */
#define _LCD_CURSOR_LEFT 0x10     /* bewegt den Cursor nach links */

#define SPALTEN 16                /* Spaltenanzahl des LCDs */
#define ZEILEN 2                  /* Zeilenanzahl des LCDs */

/*************************************************************************************/
/* globale Variablendeklaration                                                      */
/*************************************************************************************/

sbit RS = P0^5;                     /* Register select: 0 = Instruction
                                       register / 1 = Data register) */
sbit RW = P0^6;                     /* Read (1) / Write (0) select */
sbit EN = P0^7;                     /* LCD enable (1) / disable (0) */
sbit D0 = P2^0;
sbit D1 = P2^1;
sbit D2 = P2^2;
sbit D3 = P2^3;
sbit D4 = P2^4;
sbit D5 = P2^5;
sbit D6 = P2^6;
sbit D7 = P2^7;
sbit BL = P4^0;                     /* Backlight des LC-Displays (0) = an */

/*************************************************************************************/
/* Initialisieren des LC-Displays                                                    */
/*************************************************************************************/

void init_LCD (void)
{
  w_s_t(25);                                /* Initialisierung laut Datenblatt */
  RS = 0; RW = 0; P2 = 0x30;
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(10);
  RS = 0; RW = 0; P2 = 0x30;
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(10);
  RS = 0; RW = 0; P2 = 0x30;
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
                                            /* Initialisierung ENDE */
  w_s_t (10);
  RS = 0; RW = 0; P2 = 0x38;                /*0011 1000 Function Set: 8bits interface
                                                                      2 line display
                                                                      5 x 8 dots */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(1);
  RS = 0; RW = 0; P2 = 0x01;                /* lscht die Anzeige */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(1);
  RS = 0; RW = 0; P2 = _LCD_CURSOR_RIGHT;   /* bewegt den Cursor nach rechts */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(1);
  RS = 0; RW = 0; P2 = 0x0C;                /* Display an, Cursor aus, blinken aus */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  w_s_t(1);
  RS = 0; RW = 0; P2 = 0x02;                /* Adresszhler auf 0x00 */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  BL = 0;                                   /* Backlight ein (0) / aus (1)*/
  printf ("LC-Display initialisiert...\n");
} /* init_LCD */

/*************************************************************************************/
/* LCD "busy"-check                                                                  */
/*************************************************************************************/

void busy_LCD (void)
{
  unsigned int ui_i = 1;
  unsigned int ui_busy = 1;
//  printf ("starte busy_LCD...\n");
  RS = 0;                                   /* 0 = Instruction register */
  RW = 1;                                   /* Read select */
  do
  {
    EN = 0; EN = 1; w_s_t(3); EN = 0;       /* H -> L bernimmt Einstellungen */
    printf ("busy_LCD loop: %u\n", ui_i);   /* zur Hilfe, ob busy_LCD "hngt" */
    ui_i++;
    ui_busy = D7;
    EN = 0; EN = 1; w_s_t(1); EN = 0;       /* H -> L bernimmt Einstellungen */
  }
  while (ui_busy == 1);
//  printf ("busy_LCD beendet...\n");
} /* busy_LCD */

/*************************************************************************************/
/* Display an, Cursor aus, blinken aus                                               */
/*************************************************************************************/

void on_LCD (void)
{
//  printf ("starte on_LCD...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x0C;                                /* Display an, Cursor aus, blinken aus */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  BL = 0;
//  printf ("on_LCD beendet...\n");
} /* on_LCD */

/*************************************************************************************/
/* Display aus, Cursor aus, blinken aus                                              */
/*************************************************************************************/

void off_LCD (void)
{
//  printf ("starte on_LCD...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x08;                                /* Display aus, Cursor aus, blinken aus */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  BL = 1;
//  printf ("on_LCD beendet...\n");
} /* off_LCD */

/*************************************************************************************/
/* Anzeige lschen                                                                   */
/*************************************************************************************/

void clear_LCD (void)
{
//  printf ("starte clear_LCD...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x01;                                /*lscht die Anzeige */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
//  printf ("clear_LCD beendet...\n");
} /* clear_LCD */


/*************************************************************************************/
/* Cursor zurckstellen                                                              */
/*************************************************************************************/

void home_LCD (void)
{
//  printf ("starte home_LCD...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x02;                                /* Adresszhler auf 0x00 */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
//  printf ("home_LCD beendet...\n");
} /* home_LCD */

/*************************************************************************************/
/* Anzeige lschen und Cursor zurckstellen                                          */
/*************************************************************************************/

void clear_home_LCD (void)
{
//  printf ("starte clear_home_LCD...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x01;                                /* lscht die Anzeige */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  busy_LCD();
  RS = 0; RW = 0;
  P2 = 0x02;                                /* Adresszhler auf 0x00 */
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
//  printf ("clear_home_LCD beendet...\n");
} /* clear_home_LCD */

/*************************************************************************************/
/* Display an, Cursor an, blinken an                                                 */
/*************************************************************************************/

void blink_cursor (void)
{
//  printf ("starte blink_Cursor...\n");
  busy_LCD();
  RS = 0; RW = 0;
  P2 = _LCD_BLINK_CURSOR;
  EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
//  printf ("blink_Cursor beendet...\n");
} /* blink_Cursor */

/*************************************************************************************/
/* bewegt Displayinhalt ein Zeichen nach links                                       */
/*************************************************************************************/

void shift_left_LCD (unsigned int ui_stellen)
{
//  printf ("starte shift_left_LCD...\n");
  unsigned int ui_i;
  for (ui_i = 0; ui_i < ui_stellen; ui_i++)
  {
    busy_LCD();
    RS = 0; RW = 0;
    P2 = _LCD_SHIFT_LEFT;
    EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  }
//  printf ("shift_left_LCD beendet...\n");
} /* shift_left_LCD */

/*************************************************************************************/
/* bewegt Displayinhalt ein Zeichen nach rechts                                      */
/*************************************************************************************/

void shift_right_LCD (unsigned int ui_stellen)
{
//  printf ("starte shift_right_LCD...\n");
  unsigned int ui_i;
  for (ui_i = 0; ui_i < ui_stellen; ui_i++)
  {
    busy_LCD();
    RS = 0; RW = 0;
    P2 = _LCD_SHIFT_RIGHT;
    EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  }
//  printf ("shift_right_LCD beendet...\n");
} /* shift_right_LCD */

/*************************************************************************************/
/* bewegt den Cursor ein Zeichen nach rechts                                         */
/*************************************************************************************/

void right_cursor (unsigned int ui_stellen)
{
//  printf ("starte right_cursor...\n");
  unsigned int ui_i;
  for (ui_i = 0; ui_i < ui_stellen; ui_i++)
  {
    busy_LCD();
    RS = 0; RW = 0;
    P2 = _LCD_CURSOR_RIGHT;
    EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  }
//  printf ("right_cursor beendet...\n");
} /* right_cursor */

/*************************************************************************************/
/* bewegt den Cursor ein Zeichen nach links                                          */
/*************************************************************************************/

void left_cursor (unsigned int ui_stellen)
{
//  printf ("starte left_cursor...\n");
  unsigned int ui_i;
  for (ui_i = 0; ui_i < ui_stellen; ui_i++)
  {
    busy_LCD();
    RS = 0; RW = 0;
    P2 = _LCD_CURSOR_LEFT;
    EN = 0; EN = 1; EN = 0;                   /* H -> L bernimmt Einstellungen */
  }
//  printf ("left_cursor beendet...\n");
} /* left_cursor */

/*************************************************************************************/
/* Zeichen (uc-zeichen) an LCD senden                                                */
/*************************************************************************************/

void char_LCD (unsigned char uc_char)
{
  busy_LCD();
  RS = 1; RW = 0;                               /* Data, Write */
  P2 = uc_char;
  EN = 0; EN = 1; EN = 0;                       /* H -> L bernimmt Einstellungen */
} /* char_LCD */

/*************************************************************************************/
/* String an LCD senden (Textausgabe)                                                */
/* nach dem 16. Zeichen wird der Displayinhalt um ein Zeichen nach links verschoben  */
/*************************************************************************************/

void string_LCD (const char *__putstr)
{
  unsigned int ui_i = 0;
  char __ps;

  __ps = *__putstr;
  while (__ps > 0 && ui_i < 16)
  {
    __putstr++;
    if (__ps == 0)
    {
      break;
    } /* if */
    char_LCD(__ps);
    __ps = *__putstr;
    ui_i++;
  } /* while */
  while (__ps > 0 && ui_i >= 16)
  {
    __putstr++;
    if (__ps == 0)
    {
      break;
    } /* if */
    w_s_t(100);
    shift_left_LCD(1);       /* Displayinhalt ein Zeichen nach links verschieben */
    w_s_t(15);
    char_LCD(__ps);
    __ps = *__putstr;
    ui_i++;
  } /* while */
} /* string_LCD */

#endif /* _LCD_H_ */